# taken from https://github.com/opencobra/cobrapy/blob/devel/appveyor/build_glpk.py

import os
import sys
# import hashlib
import tarfile
import struct
import shutil
import setuptools.msvc
try:
    import urllib2
except ImportError:  # python 3
    import urllib.request as urllib2

# these need to be set to the latest glpk version
glpk_version = os.getenv('NEW_GLPK_VERSION')
# glpk_md5 = "eda7965907f6919ffc69801646f13c3e"

glpk_build_dir = "glpk_build/glpk-%s" % glpk_version
url = "http://ftp.gnu.org/gnu/glpk/glpk-%s.tar.gz" % glpk_version
bitness = struct.calcsize("P") * 8


# def md5(fname):
#     hash = hashlib.md5()
#     with open(fname, "rb") as f:
#         for chunk in iter(lambda: f.read(4096), b""):
#             hash.update(chunk)
#     return hash.hexdigest()


def get_vcvarsall_cmd():
    py_ver = sys.version_info
    if py_ver.major == 3 and py_ver.minor >= 5:
        vc_ver = 14
    elif py_ver.major == 3 and py_ver.minor >= 3:
        vc_ver = 10
    else:
        vc_ver = 9
    vc_path = setuptools.msvc.msvc9_find_vcvarsall(vc_ver)
    assert vc_path is not None
    return '"%s" %s' % (vc_path, " amd64" if bitness == 64 else "")


if not os.path.isdir("glpk_build/"):
    os.mkdir("glpk_build")
if not os.path.isdir(glpk_build_dir):
    response = urllib2.urlopen(url)
    with open("glpk-download.tar.gz", "wb") as outfile:
        outfile.write(response.read())
    # assert md5("glpk-download.tar.gz") == glpk_md5
    with tarfile.open("glpk-download.tar.gz") as infile:
        infile.extractall("glpk_build")

os.chdir("%s/w%d" % (glpk_build_dir, bitness))
if not os.path.isfile("glpk.lib"):
    shutil.copy2("config_VC", "config.h")
    os.system(get_vcvarsall_cmd() + "& nmake /f Makefile_VC")
shutil.copy2("glpk.lib", "../../..")
os.chdir("../../..")
shutil.copy2(glpk_build_dir + "/src/glpk.h", ".")



# swiglpk - Swig Python bindings for the GNU Linear Programming Kit (GLPK)
# Copyright (C) 2015 The Novo Nordisk Foundation Center for Biosustainability
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import nose
from swiglpk import *

def test_swiglpk():
    nose.tools.assert_equal(1, 1)
    # print(glp_version())
    # ia = intArray(1+1000); ja = intArray(1+1000);
    # ar = doubleArray(1+1000);
    # lp = glp_create_prob();
    # glp_set_prob_name(lp, "sample");
    # glp_set_obj_dir(lp, GLP_MAX);
    # glp_add_rows(lp, 3);
    # glp_set_row_name(lp, 1, "p");
    # glp_set_row_bnds(lp, 1, GLP_UP, 0.0, 100.0);
    # glp_set_row_name(lp, 2, "q");
    # glp_set_row_bnds(lp, 2, GLP_UP, 0.0, 600.0);
    # glp_set_row_name(lp, 3, "r");
    # glp_set_row_bnds(lp, 3, GLP_UP, 0.0, 300.0);
    # glp_add_cols(lp, 3);
    # glp_set_col_name(lp, 1, "x1");
    # glp_set_col_bnds(lp, 1, GLP_LO, 0.0, 0.0);
    # glp_set_obj_coef(lp, 1, 10.0);
    # glp_set_col_name(lp, 2, "x2");
    # glp_set_col_bnds(lp, 2, GLP_LO, 0.0, 0.0);
    # glp_set_obj_coef(lp, 2, 6.0);
    # glp_set_col_name(lp, 3, "x3");
    # glp_set_col_bnds(lp, 3, GLP_LO, 0.0, 0.0);
    # glp_set_obj_coef(lp, 3, 4.0);
    # ia[1] = 1; ja[1] = 1; ar[1] = 1.0; # a[1,1] = 1
    # ia[2] = 1; ja[2] = 2; ar[2] = 1.0; # a[1,2] = 1
    # ia[3] = 1; ja[3] = 3; ar[3] = 1.0; # a[1,3] = 1
    # ia[4] = 2; ja[4] = 1; ar[4] = 10.0; # a[2,1] = 10
    # ia[5] = 3; ja[5] = 1; ar[5] = 2.0; # a[3,1] = 2
    # ia[6] = 2; ja[6] = 2; ar[6] = 4.0; # a[2,2] = 4
    # ia[7] = 3; ja[7] = 2; ar[7] = 2.0; # a[3,2] = 2
    # ia[8] = 2; ja[8] = 3; ar[8] = 5.0; # a[2,3] = 5
    # ia[9] = 3; ja[9] = 3; ar[9] = 6.0; # a[3,3] = 6
    # glp_load_matrix(lp, 9, ia, ja, ar);
    # glp_simplex(lp, None);
    # Z = glp_get_obj_val(lp);
    # x1 = glp_get_col_prim(lp, 1);
    # x2 = glp_get_col_prim(lp, 2);
    # x3 = glp_get_col_prim(lp, 3);
    # print("\nZ = %g; x1 = %g; x2 = %g; x3 = %g\n" % (Z, x1, x2, x3))
    #
    # nose.tools.assert_almost_equal(Z, 733.3333333333333)
    # nose.tools.assert_almost_equal(x1, 33.333333333333336)
    # nose.tools.assert_almost_equal(x2, 66.66666666666666)
    # nose.tools.assert_almost_equal(x3, 0)
    #
    # glp_delete_prob(lp);



from __future__ import print_function
import re
import requests

response = requests.get("http://ftp.gnu.org/gnu/glpk/", timeout=30)
if response.ok:
    major, minor = 0, 0
    for item in set(re.findall('glpk-\d+\.\d+\.tar\.gz', response.text)):
        match = re.findall('(\d+)\.(\d+)', item)
        assert len(match) == 1
        assert len(match[0]) == 2
        new_major, new_minor = int(match[0][0]), int(match[0][1])
        if new_major > major:
            major = new_major
        if new_minor > minor:
            minor = new_minor

    print('{}.{}'.format(major, minor), end='')
else:
    print("Couldn't reaction GNU FTP server. Status code {}".format(response.status))




from __future__ import print_function
import requests

try:
    from packaging.version import parse
except ImportError:
    from pip._vendor.packaging.version import parse


URL_PATTERN = 'https://pypi.python.org/pypi/{package}/json'


def get_version(package, url_pattern=URL_PATTERN):
    """Return version of package on pypi.python.org using json. Adapted from https://stackoverflow.com/a/34366589"""
    req = requests.get(url_pattern.format(package=package))
    version = parse('0')
    if req.status_code == requests.codes.ok:
        # j = json.loads(req.text.encode(req.encoding))
        j = req.json()
        releases = j.get('releases', [])
        for release in releases:
            ver = parse(release)
            if not ver.is_prerelease:
                version = max(version, ver)
    return version

version = get_version('swiglpk')
major, minor, patch = version.base_version.split('.')
print('{}.{}'.format(major, minor), end='')


# swiglpk - Swig Python bindings for the GNU Linear Programming Kit (GLPK)
# Copyright (C) 2015 The Novo Nordisk Foundation Center for Biosustainability
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# http://stackoverflow.com/questions/12491328/python-distutils-not-include-the-swig-generated-module

import os
from distutils.core import setup, Extension
from distutils.command.build import build
import subprocess
import versioneer

def copy_glpk_header():
    if os.path.isfile('glpk.h'):
        print('glpk.h found in source directory')
        glpk_header_path = os.path.join('./', 'glpk.h')
    else:
        print('Trying to determine glpk.h location')
        glpsol_path = os.path.dirname(subprocess.check_output(['which', 'glpsol']))
        glpk_header_path = os.path.join(os.path.dirname(glpsol_path).decode("utf-8"), 'include', 'glpk.h')
        print('glpk.h found at {}'.format(glpk_header_path))
    if os.path.exists(glpk_header_path):
        with open('swiglpk/glpk_clean.h', 'w') as out_handle:
            with open(glpk_header_path) as in_handle:
                for line in in_handle:
                    if line == 'void glp_vprintf(const char *fmt, va_list arg);\n':
                        out_handle.write('// The following line is commented out because it is causing problems with swig\n')
                        out_handle.write('// void glp_vprintf(const char *fmt, va_list arg);')
                    else:
                        out_handle.write(line)
    else:
        raise Exception('Could not find glpk.h! Maybe glpk or glpsol is not installed.')

try:
    with open('README.rst', 'r') as f:
        long_description = f.read()
except:
    long_description = ''

# Copy and process glpk.h into current directory
copy_glpk_header()


custom_cmd_class = versioneer.get_cmdclass()

class CustomBuild(build):
    sub_commands = [
        ('build_ext', build.has_ext_modules),
        ('build_py', build.has_pure_modules),
        ('build_clib', build.has_c_libraries),
        ('build_scripts', build.has_scripts),
    ]

custom_cmd_class['build'] = CustomBuild

try:
    from wheel.bdist_wheel import bdist_wheel

    class CustomBdistWheel(bdist_wheel):
        def run(self):
            self.run_command('build_ext')
            bdist_wheel.run(self)

    custom_cmd_class['bdist_wheel'] = CustomBdistWheel
except ImportError:
    pass  # custom command not needed if wheel is not installed

setup(
    name='swiglpk',
    version=versioneer.get_version(),
    cmdclass=custom_cmd_class,
    packages=['swiglpk'],
    package_dir={'swiglpk': 'swiglpk'},
    author='Nikolaus Sonnenschein',
    author_email='niko.sonnenschein@gmail.com',
    description='swiglpk - Simple swig bindings for the GNU Linear Programming Kit',
    license='GPL v3',
    keywords='optimization swig glpk',
    url='https://github.com/biosustain/swiglpk',
    long_description=long_description,
    test_suite='nose.collector',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Topic :: Scientific/Engineering',
        'Topic :: Software Development',
        'Intended Audience :: Science/Research',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.4',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)'
    ],
    ext_modules=[Extension("swiglpk._swiglpk",
                           sources=["swiglpk/glpk.i"],
                           libraries=['glpk'])],
    include_package_data=True
)
